package require tdom

oo::class create P100GCapture {
	variable captureDoc
	variable captureFilename
	variable noiseTransferTotalDuration
	variable port1Interface
	variable port2Interface
	variable extFreqRef
	variable cdfFilename
	variable noiseTransferRecoveryTime
	variable noiseTransferGuardTime
	
    constructor { filepath filename } {
		# Open the capture file and extract the xml header
		# Store the header in a file
		# Read the file into captureDoc for later use
		#puts $filepath
		#puts $filename
        set captureFilename "$filepath/$filename"
		set xmlHeaderFilename "$filepath/header.xml"
		set noiseTransferTotalDuration -1
		set cdfFilename $filename
		set port1Interface ""
		set port2Interface ""
		set extFreqRef ""
		set noiseTransferRecoveryTime "UNDEFINED"
		set noiseTransferGuardTime "UNDEFINED"
		
		# Open the capture file and get the xml header
		set fin [open $captureFilename]
		set fout [open $xmlHeaderFilename "w+"]
	
		extractHeader $fin $fout
		#puts "Finding data..."
		#findData $fin
	
		close $fin
		close $fout
	
		# Parse the xml
		set xmlFid [open $xmlHeaderFilename]
		set captureDoc [dom parse [read $xmlFid]]
		close $xmlFid
		
		my getCommonSettings
	}

    destructor {}
	
	method getCaptureFilename {} {
		return "$captureFilename"
	}

	method isNoiseTransfer {} {
		set retval False
		set root [$captureDoc documentElement]
		set node /header/settings/noiseTransfer
		set n [$root selectNodes $node]
		if {$n != ""} {set retval True}
		return $retval
	}
	method isMasterSlave {} {
		set retval False
		set root [$captureDoc documentElement]
		set node /header/settings/masterSlave
		set n [$root selectNodes $node]
		if {$n != ""} {set retval True}
		return $retval
	}
	
	method getNoiseTransferStimulusList { } {
		set totalDuration 0
		set root [$captureDoc documentElement]
		set noiseTransferNode /header/settings/noiseTransfer/tone
		set toneList [$root selectNodes $noiseTransferNode]
		#puts $toneList
		set stimList ""
		foreach tone $toneList {
			set freq [[$tone selectNodes frequency/text()] nodeValue]
			set freq [string trim $freq]
			set ampl [[$tone selectNodes amplitude/text()] nodeValue]
			set ampl [string trim $ampl]
			set duration [[$tone selectNodes duration/text()] nodeValue]
			set duration [string trim $duration]
			set noiseTransferRecoveryTime [[$tone selectNodes settling/text()] nodeValue]
			set noiseTransferRecoveryTime [string trim $noiseTransferRecoveryTime]
			set noiseTransferGuardTime [[$tone selectNodes guard/text()] nodeValue]
			set noiseTransferGuardTime [string trim $noiseTransferGuardTime]
			set totalDuration [expr $totalDuration + $duration]
			#set step [list "$freq $ampl $duration $settling $guard"]
			set step "$freq $ampl $duration $noiseTransferRecoveryTime $noiseTransferGuardTime"
			#puts $step
			set stimList [lappend stimList $step]
		}
		#puts $stimList
		set noiseTransferTotalDuration $totalDuration
		return $stimList
	}
	
	method getNoiseTransferStimulus {} {
		set root [$captureDoc documentElement]
		set node /header/settings/noiseTransfer/noiseTarget/text()
		set val [[$root selectNodes $node] nodeValue]
		return [string trim $val]
	}

	
	method getNoiseTransferTotalDuration {} {
		return $noiseTransferTotalDuration
	}
	method getNoiseTransferRecoveryTime {} {
		return $noiseTransferRecoveryTime
	}
	method getNoiseTransferGuardTime {} {
		return $noiseTransferGuardTime
	}

	method getCommonSettings { } {
		set root [$captureDoc documentElement]
		set node /header/settings/commonSetting
		set nodeList [$root selectNodes $node]
		set valList ""
		foreach n $nodeList {
			set label [$n getAttribute label]
			set val [[$n selectNodes text()] nodeValue]
			set val [string trim $val]
			#set step [list "$freq $ampl 16.0 $duration"]
			#puts $val
			#set stimList [append stimList $step]
			if {$label == "Ethernet Port1"} {set port1Interface $val}
			if {$label == "Ethernet Port2"} {set port2Interface $val}
			if {$label == "Ref Clk In"} 	{set extFreqRef $val}
		}
		#puts $stimList
		#set noiseTransferTotalDuration $totalDuration
		#return $stimList
	}
	
	method getPort1Interface {} { return $port1Interface}
	method getPort2Interface {} { return $port2Interface}
	method getExternalFrequencyReference {} { return $extFreqRef}

	
	method getSetting {setting} {
		set root [$captureDoc documentElement]
		set node /header/settings/$setting/text()
		set val [[$root selectNodes $node] nodeValue]
		return [string trim $val]
	}
	
	method getExternalCalibration {} {
		set root [$captureDoc documentElement]
		set node /header/settings/externalCalibration/text()
		set val [[$root selectNodes $node] nodeValue]
		return [string trim $val]
	}
	method getDataRecord {} {
		set root [$captureDoc documentElement]
		set node /header/settings/dataRecord/text()
		set val [[$root selectNodes $node] nodeValue]
		return [string trim $val]
	}
	
	# Take a print function as an argument. This allows the output to be re-directed as the caller wishes
	method printHeaderGeneral {p} {
		$p "--- General Info ----------------------------------------------------------------------------"
		# Instrument details
		set id "Type: [my getSetting instrumentType]\t Name: [my getSetting instrumentName]\t SN: [my getSetting serial]\t SW: [my getSetting softwareVersion]"
		$p $id
		# Common Settings
		set cs "Port1: $port1Interface\t Port2: $port2Interface\t Ref: $extFreqRef"
		$p $cs
		
	}
	
	method printHeaderNoiseTransfer {p} {
		$p "--- Noise Transfer Info ---------------------------------------------------------------------"
		if {[my isNoiseTransfer]} {
			set s "In: [my getNoiseTransferStimulus]\t Out: [my getSetting dataRecord]"
			$p $s
			set s "Freq\t\tAmpl\tDuration\tSettling\tGuard"
			$p $s
			foreach step [my getNoiseTransferStimulusList] {
				set s [format "%.8f\t%.2f\t%4.8f\t%.2f\t\t%.2f" [lindex $step 0] [lindex $step 1] [lindex $step 2] [lindex $step 3] [lindex $step 4]]
				$p $s
			}
		} else {
			$p "Capture file does not contain a Noise Transfer measurement"
		}
	}
	
	method validateStimulus {p expectedStimulus} {
		set captureStimulus [my getNoiseTransferStimulus]
		if {$expectedStimulus == "PTP"} {
			if {$captureStimulus != "PTP"} {
					$p "$cdfFilename is not a capture from a PTP-XXX Noise Transfer measurement."
					return FALSE
			}
		} elseif {$expectedStimulus == "SYNCE"} {
			if {$captureStimulus != "SyncE"} {
					$p "$cdfFilename is not a capture from a SYNCE-XXX Noise Transfer measurement."
					return FALSE
			}
		} else {
			$p "validateStimulus: Expected stimulus $expectedStimulus not recognised"
			return FALSE
		}
		return TRUE
	}
	
	method validateMeasurement {p expectedMeasurement} {
		set expected ""
		set captureMeasurement [my getDataRecord]
		if {$expectedMeasurement == "PTP"} {set expected "Ptp1588"}
		if {$expectedMeasurement == "1PPS"} {set expected "OnePpsAccuracy"}
		if {$captureMeasurement != $expected} {
			log "$cdfFilename is not a capture from a XXX-$expectedMeasurement Noise Transfer measurement."
			return FALSE
		}
		return TRUE
	}
}

###### Helper functions ##############
proc readTag {fid} {
	set c ''
	set tag ""
	while {$c != "<"} {
		set c [read $fid 1]		
	}
	while  {$c != ">"} {
		set c [read $fid 1]
		append tag $c
	}
	return "<$tag"
}

proc readToTagEnd {fid} {
	set c ''
	set tag ""
	while  {$c != ">"} {
		set c [read $fid 1]
		append tag $c
	}
	return $tag
}

proc extractHeader {fin fout} {
	set tag ""

	while {$tag != "</header>"} {
		set tag [readToTagEnd $fin]
		#puts $tag
		puts $fout $tag
	}
}

proc findData {fin} {
	set tag ""

	while {$tag != "</data>"} {
		set tag [readTag $fin]
		puts $tag
	}
	puts "Found <\data>"
	set d [read $fin 1]
	puts $d
}

proc exampleTestCode {} {
	P100GCapture create capture "./ReProcessP100G/" xmltest.cdf

	puts [capture getSetting serial]
	puts [capture getSetting softwareVersion]
	puts [capture getSetting portReference]
	puts [capture getSetting instrumentType]
	puts [capture getSetting instrumentName]
	puts [capture getSetting startTime]
	puts [capture getSetting period]
	#capture getCommonSettings
	puts [capture getPort1Interface]
	puts [capture getPort2Interface]
	puts [capture getExternalFrequencyReference]
	capture printHeaderGeneral {puts}

	puts "MSE: [capture isMasterSlave]"
	puts [capture getSetting "port1IngressDelay1"]
	puts [capture getSetting "port1IngressDelay2"]
	puts [capture getSetting "port1EgressDelay1"]
	puts [capture getSetting "port1EgressDelay2"]
	puts [capture getSetting "port2IngressDelay1"]
	puts [capture getSetting "port2IngressDelay2"]
	puts [capture getSetting "port2EgressDelay1"]
	puts [capture getSetting "port2EgressDelay2"]
	
	if { [capture isNoiseTransfer] } {
		set sList [capture getNoiseTransferStimulusList]
		#set sList [processP100GCapture xmltest.cdf]
		puts $sList

		puts [capture getNoiseTransferTotalDuration]
		#puts [capture getExternalCalibration]
		puts [capture getDataRecord]

		capture printHeaderNoiseTransfer {puts}
	}

	capture destroy
}

#exampleTestCode