import sys, os, logging
import getopt
import argparse
from time import strftime,gmtime

from Defaults import *
import Globals as gvar



#################################################################################################
## Command line options This contains the full list of keys (setting names) that are valid.
## 
#################################################################################################

validKeyDict = {'InstrIp','GuiIp','GuiPort','Type','PtpPoints','SyncEPoints','CableDelay','LineRate','Interface','Reference',
                'TwoStep','SettlingTime','RecoveryTime','1ppsThreshold','1ppsImpedance','LimitsEnabled','AutoNegotiate','SyncE','RemoteCaptureFolder',
                'RemoteProfileFolder','Debug','ReProcess','Filter','SimulatedData','Graphing','GraphingYAxis','MasterMcastAddress',
                'SlaveMcastAddress','Port1Config', 'Port2Config'}


## These keys are not relevant when re-procesing data and so are marked as NA on the UI
reprocessIgnore = {'CableDelay','LineRate','Interface','Reference','SettlingTime','1ppsThreshold','1ppsImpedance','AutoNegotiate','SyncE',}

## These settings are not shown on the UI
reprocessOmit = {'RemoteCaptureFolder','RemoteProfileFolder','Debug','GuiPort','Graphing'}


################################################################################################
## Check a settings file for different errors
## Returns to the caller if everything is OK; otherwise will terminate the script
#################################################################################################


revisit = 1


def checkSettingsFile(filename, validKeys, defaultConfig):

    # Argparse has already checked for file existing
    inputDictionary ={}
    try:
        with open (filename) as f:
            exec (f.read(), inputDictionary)
    except:
        logging.exception("Python syntax error in file (%s)" % filename)
        exit()


    # File has been parsed OK. Now check that it contains the expected dictionary

    
    if defaultConfig and len (inputDictionary['defaults']) == 0:
        print ("ERROR: File defaults.py not contain a dictionary named 'defaults'")
        sys.exit()
    elif defaultConfig == False and len(inputDictionary['userDefinedConfig']) == 0:
        print ("Error: File %s does not contain a dictionary named 'userDefinedConfig"% filename)
        sys.exit()

    # Check it contains only valid keys
    keys = {}

    if defaultConfig:
        keys = inputDictionary['defaults']
    else: 
        keys = inputDictionary['userDefinedConfig']

    # Check for invalid keys
    for key in keys:
        if key not in validKeys:
            print ("Unrecognised key (%s) in file (%s)  , exiting program" % (key, filename))
            sys.exit()
    
    if defaultConfig:
        return inputDictionary['defaults']
    else:
        return inputDictionary['userDefinedConfig']



##------------------------------------------------------------------------------
def parseSettingsFile(dictionaryName):

    if 'InstrIp' in dictionaryName.keys():
        
        validIP = CheckIPAddress(dictionaryName['InstrIp'], False)  
        if validIP == True:
            gvar.parameters['InstrIp'] = dictionaryName['InstrIp']
            print ("Instrument IP address = %s"% gvar.parameters['InstrIp'])
        else:
            print ("The IP address is not valid: %s" % dictionaryName['InstrIp'])
            sys.exit()


    if 'GuiIp' in dictionaryName.keys():

        validIP = CheckIPAddress(dictionaryName['GuiIp'], True)  
        if validIP == True:
            gvar.parameters['GuiIp'] = dictionaryName['GuiIp']
            print ("Gui IP address = %s"% gvar.parameters['GuiIp'])
        else:
            print ("The Gui IP address is not valid: %s" % dictionaryName['GuiIp'])
            sys.exit()

    if 'GuiPort' in dictionaryName.keys():
        if (dictionaryName['GuiPort'] > 0):
            gvar.parameters['GuiPort'] = dictionaryName['GuiPort']
            print("PX GUI Port = %d" % gvar.parameters['GuiPort'])
        else:
            print("PX GUI Port: %d is NOT a valid TCP Port" % dictionaryName['GuiPort'])
            sys.exit()        

    if 'Type' in dictionaryName.keys():
        if dictionaryName['Type'] in ('PTP-BOTH', 'PTP-1PPS', 'PTP-PTP', 'SyncE-BOTH', 'SyncE-1PPS', 'SyncE-PTP'):
            gvar.parameters['Type'] = dictionaryName['Type']
            print ("Measurement Type %s" %gvar.parameters['Type'])
        else:
            print("ERROR: Invalid Type parameter in Config file : %s" % dictionaryName['Type'])
            sys.exit()
      
    if 'PtpPoints' in dictionaryName.keys():

        outPoints = []
        validPoints = CheckSequencePoints(dictionaryName['PtpPoints'], 9, outPoints,True)

        if validPoints == True:
            gvar.parameters['PtpPoints'] = []
            for i in outPoints:
                gvar.parameters['PtpPoints'].append(str(i))                
            print("Selected PtpPoints = %s"%gvar.parameters['PtpPoints'])
        else:
            print (" ERROR: Non Valid PTP points in ConfigFile: %s"% dictionaryName['PtpPoints'])
            sys.exit()

    if 'SyncEPoints' in dictionaryName.keys():
        outputPoints = []
        validPoints = CheckSequencePoints(dictionaryName['SyncEPoints'],11, outputPoints,True)

        if validPoints == True:
            gvar.parameters['SyncEPoints'] = [] 
            for i in outputPoints:
                gvar.parameters['SyncEPoints'].append(str(i))                
            print("Selected SyncEPoints = %s"%gvar.parameters['SyncEPoints'])
        else:
            print (" ERROR: Non Valid SyncE points in Config File: %s"% dictionaryName['SyncEPoints'])
            sys.exit()            
##############################################################################################################################   HERE
    if 'CableDelay' in dictionaryName.keys():

        if (dictionaryName['CableDelay']>= 0 ) and (dictionaryName['CableDelay'] <= 1000):
            gvar.parameters['CableDelay'] = dictionaryName['CableDelay']
            print ("Cable Delay (in ns) = %d" % gvar.parameters['CableDelay'])
        else:
            print ("ERROR: Cable Delay (%d)  should be in range 0..1000"% dictionaryName['CableDelay'] )
            sys.exit()
        
    if 'LineRate' in dictionaryName.keys():

        if dictionaryName['LineRate'] in ('1GBE', '100BASET', '10GBE'):
            gvar.parameters['LineRate'] = dictionaryName['LineRate']
            print ("Line interface rate %s " % gvar.parameters['LineRate'])
        else:
            print ("ERROR: LineRate (%s)  should be one of '1GBE', '100BASET', '10GBE' "% dictionaryName['LineRate'] )
            sys.exit()
    
    if 'Interface' in dictionaryName.keys(): 
        if dictionaryName['Interface'] in ('ELECTRICAL', 'OPTICAL', 'XFP', 'SFPPLUS'):
            gvar.parameters['Interface'] = dictionaryName['Interface']
            print("Interface type selection %s" % gvar.parameters['Interface'])
        else:
            print ("ERROR: Interface (%s)  should be one of 'ELECTRICAL', 'OPTICAL', 'XFP', 'SFPPLUS' "% dictionaryName['Interface'] )
            sys.exit()       
           

    if 'Reference' in dictionaryName.keys(): 
        if dictionaryName['Reference'] in ('EXT_10M', 'EXT_R75_E1', 'EXT_R100_T1', 'EXT_R100_E1', 'EXT_K64', 'INT'):
            gvar.parameters['Reference'] = dictionaryName['Reference']
            print ("Source of frequency reference %s " % gvar.parameters['Reference'] )
        else:
            print("ERROR: Reference (%s) must be one of 'EXT_10M', 'EXT_R75_E1', 'EXT_R100_T1', 'EXT_R100_E1', 'EXT_K64', 'INT'" % dictionaryName['Reference'])
            sys.exit()
        
    if 'TwoStep' in dictionaryName.keys():
        if dictionaryName['TwoStep'] in  ('False', 'FALSE', 'True', 'TRUE'):
            gvar.parameters['TwoStep'] = dictionaryName['TwoStep']
            print ("Select TwoStep = %s" % gvar.parameters['TwoStep'])
        else:
            print ("ERROR: TwoStep  must be either 'True' or 'False'")
            sys.exit()

    if 'SettlingTime' in dictionaryName.keys(): 
        if (dictionaryName['SettlingTime'] > 0):
            gvar.parameters['SettlingTime'] = dictionaryName['SettlingTime']
            print("Settling time = %d"% gvar.parameters['SettlingTime'])
        else:
            print ("ERROR: Settling Time (%d) is not in range 0..n" % dictionaryName['SettlingTime'])
            sys.exit()

    if 'RecoveryTime' in dictionaryName.keys(): 
        if dictionaryName['RecoveryTime'] > 0:
            gvar.parameters['RecoveryTime'] = dictionaryName['RecoveryTime']
            print ("Time (in secs) to recover = %d", dictionaryName['RecoveryTime'])
        else:
            print ("ERROR: Recovery Time (%d) is not in range 0..n" % dictionaryName['RecoveryTime'])
            sys.exit()

    if 'OneppsThreshold' in dictionaryName.keys(): 
        if (dictionaryName['OneppsThreshold'] >= 0.5) and (dictionaryName['OneppsThreshold'] <= 2.5):
            gvar.parameters['1ppsThreshold'] = dictionaryName['OneppsThreshold']
            print("Voltage threshold for 1pps measurement %f", gvar.parameters['1ppsThreshold'])
        else:
            print ("ERROR: Voltage threshold for 1pps (%f)  should be in range 0.5..2.5"% dictionaryName['OneppsThreshold'] )
            sys.exit()

## (ENT-43)

    if '1ppsImpedance' in dictionaryName.keys():  
        if dictionaryName['1ppsImpedance']  in  ('HIGH_IMPEDANCE', 'R50'):
            gvar.parameters['1ppsImpedance'] = dictionaryName['1ppsImpedance']
            print ("Set 1pps Impedance  = %s" % gvar.parameters['1ppsImpedance'])
        else:
            print ("ERROR: 1ppsImpedance must be either 'R50' or 'HIGH_IMPEDANCE'")
            sys.exit()

#

    if 'SyncE' in dictionaryName.keys():  
        if dictionaryName['SyncE']  in  ('False', 'FALSE', 'True', 'TRUE'):
            gvar.parameters['SyncE'] = dictionaryName['SyncE']
            print ("Enable or disable SyncE  = %s" % gvar.parameters['SyncE'])
        else:
            print ("ERROR: SyncE must be either 'True' or 'False'")
            sys.exit()
                
    
    if 'AutoNegotiate' in dictionaryName.keys(): 
        if dictionaryName['AutoNegotiate']  in  ('False', 'FALSE', 'True', 'TRUE'):
            gvar.parameters['AutoNegotiate'] = dictionaryName['AutoNegotiate']
            print("Enable or disable auto negotiation on inteface ports %s" % gvar.parameters['AutoNegotiate'])
        else:
            print ("ERROR: AutoNegotiate must be either 'True' or 'False'")
            sys.exit()            

    if 'RemoteCaptureFolder' in dictionaryName.keys():
        gvar.parameters['RemoteCaptureFolder'] = dictionaryName['RemoteCaptureFolder']
        print ("The folfer in which to store capture files on the remote Windows PC  -- <path/filename>")

    if 'RemoteProfileFolder' in dictionaryName.keys():
        gvar.parameters['RemoteProfileFoler'] = dictionaryName['RemoteProfileFolder']
        print("The folfer in which to obtain impairment profiles files on the remote Windows PC  -- <path/filename>")

    if 'Debug' in dictionaryName.keys():
        if dictionaryName['Debug'] in  ('False', 'FALSE', 'True', 'TRUE'):
            gvar.parameters['Debug'] = dictionaryName['Debug']
            print("Enable or disable additional debug= %s" % gvar.parameters['Debug'])
        else:
            print ("ERROR: Debug should either be True or False")
            sys.exit()

    if 'ReProcess' in dictionaryName.keys():
        if dictionaryName['ReProcess'] in  ('False', 'FALSE', 'True', 'TRUE'):
            gvar.parameters['ReProcess'] = dictionaryName['ReProcess']
            print("Reprocess previously captured data  = %s" % gvar.parameters['ReProcess'])
        else:
            print ("ERROR:  ReProcess should either be True or False ")
            sys.exit()

    if 'Filter' in dictionaryName.keys():
        if dictionaryName['Filter'] in ('LSQ', 'IIR', 'NONE'):
            gvar.parameters['Filter'] = dictionaryName['Filter']
            print ("Filter Type = %s " % gvar.parameters['Filter'])
        else:
            print ("ERROR: Filter must be one of 'LSQ', 'IIR' or 'NONE' ")
            sys.exit()

    if 'SimulatedData' in dictionaryName.keys():
        if dictionaryName['SimulatedData'] in  ('False', 'FALSE', 'True', 'TRUE'):
            gvar.parameters['SimulatedData'] = dictionaryName['SimulatedData']
            print("Simulated Data = %s" % gvar.parameters['SimulatedData'] )
        else:
            print ("ERROR: SimulatedData should be either True or False")
            sys.exit()

    if 'Graphing' in dictionaryName.keys():
        if dictionaryName['Graphing'] in  ('False', 'FALSE', 'True', 'TRUE'):
            gvar.parameters['Graphing'] = dictionaryName['Graphing']
            print ("Enable graphical UI for results %s" %gvar.parameters['Graphing'])
        else:
            print ("ERROR: Graphing should be either True or False")
            sys.exit()       

    if 'GraphingYAxis' in dictionaryName.keys():
        if dictionaryName['GraphingYAxis'] in ('GAIN', 'AMPLITUDE'):
            gvar.parameters['GraphingYAxis'] = dictionaryName['GraphingYAxis']
            print ("Plot results as %s" % gvar.parameters['GraphingYAxis'])
        else:
            print ("ERROR: GraphingYAxis must be one of 'GAIN' 'AMPLITUDE")
            sys.exit()

    if 'MasterMcastAddress' in dictionaryName.keys():
        if dictionaryName['MasterMcastAddress'] in ('FORWARDABLE', 'NONFORWARDABLE'):
            gvar.parameters['MasterMcastAddress'] = dictionaryName['MasterMcastAddress']
            print ("MasterMcastAddress %s" %gvar.parameters['MasterMcastAddress'])
        else:
            print("ERROR: Invalid MasterMcastAddress in Config file : %s" % dictionaryName['MasterMcastAddress'])
            sys.exit()

    if 'SlaveMcastAddress' in dictionaryName.keys():
        if dictionaryName['SlaveMcastAddress'] in ('FORWARDABLE', 'NONFORWARDABLE'):
            gvar.parameters['SlaveMcastAddress'] = dictionaryName['SlaveMcastAddress']
            print ("SlaveMcastAddress %s" %gvar.parameters['SlaveMcastAddress'])
        else:
            print("ERROR: Invalid SlaveMcastAddress in Config file : %s" % dictionaryName['SlaveMcastAddress'])
            sys.exit()

################################################################################################
## Check Commandline gvar.parameters
## Returns to the caller if everything is OK; otherwise will terminate the script
#################################################################################################


def commandLineParsing():


    gvar.defaults = {}

    ## The defaults settings are contained in a separate Python file. This allows the default settings to
    ## be changed without editting the main G.8273.2.py file and allows the script to be run with no arguments
    ## Make sure the file doesn't have any python errors

    gvar.defaults = checkSettingsFile ("Defaults.py" , validKeyDict, True)
    ## Defaults.py is OK - load the file and use the settings as the base gvar.parameters
    #set defaults "No defaults defined"

    gvar.parameters = gvar.defaults

    #DJWAT-AE7
    #Initialise individual --Port1Config and --Port2Config to be False
    gvar.parameters['Port1Config'] = False
    gvar.parameters['Port2Config'] = False
    gvar.parameters['LineRatePort1'] = gvar.parameters['LineRate']
    gvar.parameters['LineRatePort2'] = gvar.parameters['LineRate']
    gvar.parameters['InterfacePort1'] = gvar.parameters['Interface']
    gvar.parameters['InterfacePort2'] = gvar.parameters['Interface']
    gvar.parameters['AutoNegotiatePort1'] = gvar.parameters['AutoNegotiate']
    gvar.parameters['AutoNegotiatePort2'] = gvar.parameters['AutoNegotiate']



    #print ("\n**********  BEFORE \n")
    #print (gvar.parameters)


    if (len(sys.argv) >= 1):

		# Parse the input arguments
        parser = argparse.ArgumentParser()

        parser.add_argument ("--InstrIp",help ="The IP address of your Paragon-X instrument  <IP Addr")

        parser.add_argument ("--GuiIp",help ="IP address of the PC running the P-X GUI - 	<IP Addr> or <localhost>")
        parser.add_argument ("--GuiPort",help ="The port to use to connect to the P-X GUI	---	<integer>  : Default 9000",type= int )
        parser.add_argument ("--Type",help ="Measurement type",			choices = ('PTP-BOTH','PTP-PTP','PTP-1PPS', 'SyncE-BOTH', 'SyncE-PTP', 'SyncE-1PPS'))
        parser.add_argument ("--PtpPoints",help ="Selected frequencies one or more of range: 	--- 0-9 for PTP e.g. ['0','3','9']")
        parser.add_argument ("--SyncEPoints",help ="Selected frequencies one or more of range	--- 0-11 for SyncE e.g. ['0','3','9']")
        parser.add_argument ("--CableDelay",help = "Delay (in ns) due to cable from BC master to P-X slave  --- <integer>", type = int)
        parser.add_argument ("--LineRate",help = "Line interface rate", choices = {'1GBE', '100BASET', '10GBE'})
        parser.add_argument ("--Interface",help = "Interface type selection ( tied to LineRate)", choices = ('ELECTRICAL', 'OPTICAL', 'XFP', 'SFPPLUS'))
        parser.add_argument ("--Reference",help = "Source of frequency reference", choices =  ('EXT_10M', 'EXT_R75_E1', 'EXT_R100_T1', 'EXT_R100_E1', 'EXT_K64', 'INT'))
        parser.add_argument ("--TwoStep",help = "Select whether the device under test is a two-step clock", choices = ('TRUE', 'FALSE'))
        parser.add_argument ("--SettlingTime",help = "Time (in secs) to allow the device to settle after MSE starts           -- <integer>",type = int)

        parser.add_argument ("--RecoveryTime",help="Time (in secs) to allow the device to recover from noise starts  --- integer",type = int)
        parser.add_argument ("--OneppsThreshold", help ="Voltage threshold for 1pps measurement input --- float", type = float)      
        parser.add_argument ("--OneppsImpedance", help = "Line termination for 1pps measurement input", choices = ('R50', 'HIGH_IMPEDANCE'))

        parser.add_argument ("--SyncE", help = "Enable or disable SyncE measurement during test", choices =('TRUE', 'FALSE'))
	    
        #parser.add_argument ("-n","--AutoNegotiate", help="A")
        parser.add_argument ("--AutoNegotiate",help = "Enable or disable auto negotiation on inteface ports", choices = ('TRUE', 'FALSE'))
        parser.add_argument ("--RemoteCaptureFolder", help = "The folfer in which to store capture files on the remote Windows PC  -- <path/filename>")
        parser.add_argument ("--RemoteProfileFolder", help = "The folfer in which to store capture files on the remote Windows PC  -- <path/filename>")

        parser.add_argument ("--Debug", help = "Enable or disable additional debug information ", choices = ('TRUE', 'FALSE'))
        
        parser.add_argument ("--ReProcess",help = "Reprocess previously captured data for analysis              ** For Calnex use only",choices = ('TRUE', 'FALSE'))
        parser.add_argument ("--Filter", help = "Defines the filter type to use   ** For Calnex Use Only", choices =('IIR', 'LSQ', 'NONE'))
        parser.add_argument ("--SimulatedData", help = "Generates test data to exercise the filter  ** For Calnex Use Only",choices =('TRUE','FALSE'))
        parser.add_argument ("--Graphing", help = "Enable graphical UI for results", choices = ('TRUE','FALSE'))
        parser.add_argument ("--GraphingYAxis", help = "Plot results as either Gain(dB) or Amplitude(ns)", choices =('GAIN', 'AMPLITUDE'))
        parser.add_argument ("--ConfigFile", help = "File containing user-defined settings. THE FILE MUST EXIST --- <filename>")

        parser.add_argument ("--MasterMcastAddress", help = "Set Master Multicast Address ", choices = ('FORWARDABLE', 'NONFORWARDABLE'))
        parser.add_argument ("--SlaveMcastAddress", help = "Set Slave Multicast Address ", choices = ('FORWARDABLE', 'NONFORWARDABLE'))
        
        # DJWAT-PX-AE-7
        parser.add_argument ("--Port1Config", help = "Configuration settings for indepedent port 1  format \"LineRate,Interface, AutoNegotiate setting\"")
        parser.add_argument ("--Port2Config", help = "Configuration settings for indepedent port 2  format \"LineRate,Interface, AutoNegotiate setting\"")
 
        args = parser.parse_args()


        if args.InstrIp:

            validIP = CheckIPAddress(args.InstrIp, False)  
            if validIP == True:
                gvar.parameters['InstrIp'] = args.InstrIp
                #print ("Instrument IP address = %s"% gvar.parameters['InstrIp'])
            else:
                print ("The IP address is not valid: %s" % args.InstrIp)
                sys.exit()


        if args.GuiIp:

            validIP = CheckIPAddress(args.GuiIp, True)  
            if validIP == True:
                gvar.parameters['GuiIp'] = args.GuiIp
                #print ("Gui IP address = %s"% gvar.parameters['GuiIp'])
            else:
                print ("The Gui IP address is not valid: %s" % args.GuiIp)
                sys.exit()

        if args.GuiPort:
            if (args.GuiPort > 0):
                gvar.parameters['GuiPort'] = args.GuiPort
                #print("PX GUI Port = %d" % gvar.parameters['GuiPort'])
            else:
                print("PX GUI Port: %d is NOT a valid TCP Port" % args.GuiPort)
                sys.exit()        

        if args.Type:
            gvar.parameters['Type'] = args.Type
            #print ("Measurement Type %s" %gvar.parameters['Type'])

            # 'PTP-BOTH','PTP-PTP','PTP-1PPS', 'SyncE-BOTH', 'SyncE-PTP', 'SyncE-1PPS'))
        
        if args.PtpPoints:

            outPoints = []
            validPoints = CheckSequencePoints(args.PtpPoints, 9, outPoints,False)

            if validPoints == True:
                gvar.parameters['PtpPoints'] = []
                for i in outPoints:
                    gvar.parameters['PtpPoints'].append(str(i))                
                # print("Selected PtpPoints = %s"%gvar.parameters['PtpPoints'])
            else:
                print (" Non Valid PTP points in input: %s"% args.PtpPoints[0])
                sys.exit()

        if args.SyncEPoints:
            outputPoints = []
            validPoints = CheckSequencePoints(args.SyncEPoints,11, outputPoints,False)

            if validPoints == True:
                gvar.parameters['SyncEPoints'] = [] 
                for i in outputPoints:
                    gvar.parameters['SyncEPoints'].append(str(i))                
                #print("Selected SyncEPoints = %s"%gvar.parameters['SyncEPoints'])
            else:
                print (" Non Valid SyncE points in input: %s"% args.SyncEPoints)
                sys.exit()            

        if args.CableDelay:

            if (args.CableDelay>= 0 ) and (args.CableDelay <= 1000):
                gvar.parameters['CableDelay'] = args.CableDelay 
                #print ("Cable Delay (in ns) = %d" % gvar.parameters['CableDelay'])
            else:
                print ("ERROR: Cable Delay (%d)  should be in range 0..1000"% args.CableDelay )
                sys.exit()
        
        if args.LineRate:
            #DJWAT-AE7
            gvar.parameters['LineRate'] = args.LineRate
            #print ("Line interface rate %s " % gvar.parameters['LineRate1'])
        
        if args.Interface: 
            #DJWAT-AE7
            gvar.parameters['Interface'] = args.Interface
            #print("Interface type selection %s" % gvar.parameters['Interface'])

        if args.Reference: 
            gvar.parameters['Reference'] = args.Reference
            #print ("Source of frequency reference %s " % gvar.parameters['Reference'] )
        
        if args.TwoStep: 
            gvar.parameters['TwoStep'] = args.TwoStep
            #print ("Select TwoStep = %s" % gvar.parameters['TwoStep'])
        
        if args.SettlingTime: 
            if (args.SettlingTime > 0):
                gvar.parameters['SettlingTime'] = args.SettlingTime
                #print("Settling time = %d"% gvar.parameters['SettlingTime'])
            else:
                print ("Settling Time (%d) is not in range 0..n" % args.SettlingTime)
                sys.exit()

        if args.RecoveryTime: 
            if args.RecoveryTime > 0:
                gvar.parameters['RecoveryTime'] = args.RecoveryTime
                #print ("Time (in secs) to recover = %d", args.RecoveryTime)
            else:
                print ("Recovery Time (%d) is not in range 0..n" % args.RecoveryTime)
                sys.exit()

        if args.OneppsThreshold: 
            if (args.OneppsThreshold >= 0.5) and (args.OneppsThreshold <= 2.5):
                gvar.parameters['1ppsThreshold'] = args.OneppsThreshold
               #print("Voltage threshold for 1pps measurement %f", gvar.parameters['1ppsThreshold'])
            else:
                print ("ERROR: Voltage threshold for 1pps (%f)  should be in range 0.5..2.5"% args.OneppsThreshold )
                sys.exit()

        if args.OneppsImpedance:
            gvar.parameters['1ppsImpedance'] = args.OneppsImpedance
            #print ("Line termination for 1pps measurement %s " % gvar.parameters['1ppsImpedance'] )

        if args.SyncE:  
            gvar.parameters['SyncE'] = args.SyncE
            #print ("Enable or disable SyncE  = %s" % gvar.parameters['SyncE'])
	    
        if args.AutoNegotiate: 
            gvar.parameters['AutoNegotiate'] = args.AutoNegotiate
            #print("Enable or disable auto negotiation on inteface ports %s" % gvar.parameters['AutoNegotiate'])

        if args.RemoteCaptureFolder:
            gvar.parameters['RemoteCaptureFolder'] = args.RemoteCaptureFolder
            #print ("The folfer in which to store capture files on the remote Windows PC  -- <path/filename>")
        if args.RemoteProfileFolder:
             gvar.parameters['RemoteProfileFolder'] = args.RemoteProfileFolder
             #print("The folfer in which to store capture files on the remote Windows PC  -- <path/filename>")

        if args.Debug:
            gvar.parameters['Debug'] = args.Debug
            #print("Enable or disable additional debug= %s" % gvar.parameters['Debug'])
        
        if args.ReProcess:
            gvar.parameters['ReProcess'] = args.ReProcess
            #print("Reprocess previously captured data  = %s" % gvar.parameters['ReProcess'])

        if args.Filter:
            gvar.parameters['Filter'] = args.Filter
            #print ("Filter Type = %s " % gvar.parameters['Filter'])

        if args.SimulatedData:
            gvar.parameters['SimulatedData'] = args.SimulatedData
            #print("Simulated Data = %s" % gvar.parameters['SimulatedData'] )

        if args.Graphing:
            gvar.parameters['Graphing'] = args.Graphing
            #print ("Enable graphical UI for results %s" %gvar.parameters['Graphing'])

        if args.GraphingYAxis:
            gvar.parameters['GraphingYAxis'] = args.GraphingYAxis
            #print ("Plot results as %s" % gvar.parameters['GraphingYAxis'])

        if args.MasterMcastAddress:
            gvar.parameters['MasterMcastAddress'] = args.MasterMcastAddress
            #print ("Master Multicast Address is %s" % gvar.parameters['MasterMcastAddress'])

        if args.SlaveMcastAddress:
            gvar.parameters['SlaveMcastAddress'] = args.SlaveMcastAddress
            #print ("Slave Multicast Address is %s" % gvar.parameters['SlaveMcastAddress'])

        # DJWAT-PX-AE-7
        if args.Port1Config:

            
            # clear selection is present flag 
            gvar.parameters['Port1Config'] = False  
            
            # Parse structure to check for valid selections
            #if all OK set  gvar.parameters['Port1Config'] == True
            # plus set individual LineRatePortx InterfacePortx
            
            tmpList = []
            tmpList = args.Port1Config.split(',')
            if len(tmpList) < 3:
                print ("ERROR: --Port1Config insufficient fields, should be \"LineRate,Interface, AutoNegotiate setting\"")
                sys.exit()

            lineRate = tmpList[0]

            if lineRate == '1GBE' or lineRate == '100BASET' or lineRate == '10GBE':
                gvar.parameters['LineRatePort1'] = lineRate
            else:
                print("ERROR: --Port1Config - Invalid Line Rate: (%s) should be one of 100BASET, 1GBE, 10GBE" % lineRate)
                sys.exit()

            interface = tmpList[1]

            if interface == 'ELECTRICAL' or interface == 'OPTICAL' or interface == 'XFP' or interface == 'SFPPLUS':
                gvar.parameters['InterfacePort1'] = interface
            else:
                print("ERROR: --Port1Config - Invalid Interface (%s) should be one of ELECTRICAL,OPTICAL,XFP,SFPPLUS" %interface)
                sys.exit()

            autoNegotiate = tmpList[2]

            if autoNegotiate != 'TRUE' and autoNegotiate != 'FALSE':
                print("ERROR: --Port1Config - Invalid AutoNegotiate setting (%s) should be one of TRUE,FALSE" % autoNegotiate)
                sys.exit()
            
            gvar.parameters['AutoNegotiatePort1'] = tmpList[2]

            # set selection is present flag 
            gvar.parameters['Port1Config'] = True 

        if args.Port2Config:
 
            # clear selection is present flag 
            gvar.parameters['Port2Config'] = False  
 
            # Parse structure to check for valid selections
            #if all OK set  gvar.parameters['Port1Config'] == True
             # plus set individual LineRatePort2 InterfacePort2
            tmpList = []
            tmpList = args.Port2Config.split(',')

            if len(tmpList) < 3:
                print ("ERROR: --Port2Config insufficient fields, should be \"LineRate,Interface,AutoNegotiate setting\"")
                sys.exit()
                            
            lineRate = tmpList[0]

            if lineRate == '1GBE' or lineRate == '100BASET' or lineRate == '10GBE':
                gvar.parameters['LineRatePort2'] = lineRate
            else:
                print("ERROR: --Port2Config - Invalid Line Rate: (%s) should be one of 100BASET, 1GBE, 10GBE" % lineRate)
                sys.exit()

            interface = tmpList[1]

            if interface == 'ELECTRICAL' or interface == 'OPTICAL' or interface == 'XFP' or interface == 'SFPPLUS':
                gvar.parameters['InterfacePort2'] = interface
            else:
                print("ERROR: --Port2Config - Invalid Interface(%s)  should be one of ELECTRICAL,OPTICAL,XFP,SFPPLUS" %interface)
                sys.exit()
            
            autoNegotiate = tmpList[2]

            if autoNegotiate != "TRUE" and autoNegotiate != "FALSE":
                print("ERROR: --Port2Config - Invalid AutoNegotiate setting (%s) should be one of TRUE,FALSE" % autoNegotiate)
                sys.exit()

            
            gvar.parameters['AutoNegotiatePort2'] = tmpList[2]

            # set selection is present flag 
            gvar.parameters['Port2Config'] = True  
 
        
        
        
        if args.ConfigFile:
            userDefinedConfig = None
            if os.path.isfile(args.ConfigFile) == True :
            #    print ("Config file %s found OK" % args.ConfigFile)
                userDefinedConfig = checkSettingsFile(args.ConfigFile, validKeyDict, False)
                parseSettingsFile(userDefinedConfig)

            else:
                print("ERROR: THE CONFIG FILE MUST EXIST --- <%s>"% args.ConfigFile)
                sys.exit()

    #DJWAT-AE-7

    # check if individual port set parameters Port1Config/Port2Config have been set
    #  if not set then need to set individual Port parameters from coupled set(LineRate/Interface/Autonegotiate)
    # 
    ## if only 1 of Port1Config/Port2Config has been set then this is an error it requries portconfigs for both to work

    if (gvar.parameters['Port1Config'] == False and gvar.parameters['Port2Config'] == False):
        gvar.parameters['LineRatePort1'] = gvar.parameters['LineRate']
        gvar.parameters['LineRatePort2'] = gvar.parameters['LineRate']
        gvar.parameters['InterfacePort1'] = gvar.parameters['Interface']
        gvar.parameters['InterfacePort2'] = gvar.parameters['Interface']
        gvar.parameters['AutoNegotiatePort1'] = gvar.parameters['AutoNegotiate']
        gvar.parameters['AutoNegotiatePort2'] = gvar.parameters['AutoNegotiate']
    else:
       
        configsOK = True
        if (gvar.parameters['Port1Config'] == True  and gvar.parameters['Port2Config']== False):
            configsOK = False
        elif (gvar.parameters['Port1Config'] == False  and gvar.parameters['Port2Config']== True):
            configsOK = False

        if configsOK == False:
            print("ERROR only one of --Port1Config or --Port2Config has been used. Both must be present")
            sys.exit()



#        print ("\n**********  AFTER \n")
#        print (gvar.parameters)


##############################################################################
## Support functiond
##############################################################################

 
def CheckIPAddress(inputParam, checkLocalHost):

    # check we have xxx.xxx.xxx.xxx or localhost format
    validIP = True
    if checkLocalHost == True:
        if inputParam == 'localhost':
            return validIP

    tempList = inputParam.split('.')

    # check we have xxx.xxx.xxx.xxx or localhost format
    if len(tempList) == 4:
        for i in tempList:
            if (int(i) < 0) or (int(i) >= 255):
                validIP = False
                break

    else:
        validIP = False

    return validIP


def CheckSequencePoints(inputPoints, limitValue, outputPoints, alreadyAList):
    
    # Can enter here from a parsed file (already a list) or command line parsing ( not a list)
    if alreadyAList == True:
        tmpPoints = inputPoints.copy()
    else:
        tmpPoints = inputPoints.split(',')
    outPoints = []
    
    validPoints = True
    for i in tmpPoints:
        if i.isdigit() and int(i) >= 0 and int(i) <= limitValue:
            outPoints.append(int(i))
        else:    
            validPoints = False

    
            
    if validPoints == True and len(outPoints) > 0:
        outPoints.sort()
        outputPoints += outPoints
    return validPoints
       